/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.4.0 (build: 2013-06-26 16:27:34 UTC)
 *  on 2013-07-25 at 00:07:02 UTC 
 */

package com.google.api.services.pagespeedonline;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Pagespeedonline (v1).
 *
 * <p>
 * Lets you analyze the performance of a web page and get tailored suggestions to make that page faster.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/speed/docs/insights/v1/getting_started" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link PagespeedonlineRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Pagespeedonline extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the PageSpeed Insights API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "pagespeedonline/v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Pagespeedonline(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Pagespeedonline(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Pagespeedapi collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Pagespeedonline pagespeedonline = new Pagespeedonline(...);}
   *   {@code Pagespeedonline.Pagespeedapi.List request = pagespeedonline.pagespeedapi().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Pagespeedapi pagespeedapi() {
    return new Pagespeedapi();
  }

  /**
   * The "pagespeedapi" collection of methods.
   */
  public class Pagespeedapi {

    /**
     * Runs Page Speed analysis on the page at the specified URL, and returns a Page Speed score, a list
     * of suggestions to make that page faster, and other information.
     *
     * Create a request for the method "pagespeedapi.runpagespeed".
     *
     * This request holds the parameters needed by the the pagespeedonline server.  After setting any
     * optional parameters, call the {@link Runpagespeed#execute()} method to invoke the remote
     * operation.
     *
     * @param url The URL to fetch and analyze
     * @return the request
     */
    public Runpagespeed runpagespeed(String url) throws java.io.IOException {
      Runpagespeed result = new Runpagespeed(url);
      initialize(result);
      return result;
    }

    public class Runpagespeed extends PagespeedonlineRequest<com.google.api.services.pagespeedonline.model.Result> {

      private static final String REST_PATH = "runPagespeed";

      private final java.util.regex.Pattern URL_PATTERN =
          java.util.regex.Pattern.compile("http(s)?://.*");

      private final java.util.regex.Pattern LOCALE_PATTERN =
          java.util.regex.Pattern.compile("[a-zA-Z]+(_[a-zA-Z]+)?");

      private final java.util.regex.Pattern RULE_PATTERN =
          java.util.regex.Pattern.compile("[a-zA-Z]+");

      /**
       * Runs Page Speed analysis on the page at the specified URL, and returns a Page Speed score, a
       * list of suggestions to make that page faster, and other information.
       *
       * Create a request for the method "pagespeedapi.runpagespeed".
       *
       * This request holds the parameters needed by the the pagespeedonline server.  After setting any
       * optional parameters, call the {@link Runpagespeed#execute()} method to invoke the remote
       * operation. <p> {@link Runpagespeed#initialize(AbstractGoogleClientRequest)} must be called to
       * initialize this instance immediately after invoking the constructor. </p>
       *
       * @param url The URL to fetch and analyze
       * @since 1.13
       */
      protected Runpagespeed(String url) {
        super(Pagespeedonline.this, "GET", REST_PATH, null, com.google.api.services.pagespeedonline.model.Result.class);
        this.url = Preconditions.checkNotNull(url, "Required parameter url must be specified.");
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(URL_PATTERN.matcher(url).matches(),
              "Parameter url must conform to the pattern " +
              "http(s)?://.*");
        }
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Runpagespeed setAlt(String alt) {
        return (Runpagespeed) super.setAlt(alt);
      }

      @Override
      public Runpagespeed setFields(String fields) {
        return (Runpagespeed) super.setFields(fields);
      }

      @Override
      public Runpagespeed setKey(String key) {
        return (Runpagespeed) super.setKey(key);
      }

      @Override
      public Runpagespeed setOauthToken(String oauthToken) {
        return (Runpagespeed) super.setOauthToken(oauthToken);
      }

      @Override
      public Runpagespeed setPrettyPrint(Boolean prettyPrint) {
        return (Runpagespeed) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Runpagespeed setQuotaUser(String quotaUser) {
        return (Runpagespeed) super.setQuotaUser(quotaUser);
      }

      @Override
      public Runpagespeed setUserIp(String userIp) {
        return (Runpagespeed) super.setUserIp(userIp);
      }

      /** The URL to fetch and analyze */
      @com.google.api.client.util.Key
      private String url;

      /** The URL to fetch and analyze
       */
      public String getUrl() {
        return url;
      }

      /** The URL to fetch and analyze */
      public Runpagespeed setUrl(String url) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(URL_PATTERN.matcher(url).matches(),
              "Parameter url must conform to the pattern " +
              "http(s)?://.*");
        }
        this.url = url;
        return this;
      }

      /** Indicates if binary data containing a screenshot should be included */
      @com.google.api.client.util.Key
      private Boolean screenshot;

      /** Indicates if binary data containing a screenshot should be included [default: false]
       */
      public Boolean getScreenshot() {
        return screenshot;
      }

      /** Indicates if binary data containing a screenshot should be included */
      public Runpagespeed setScreenshot(Boolean screenshot) {
        this.screenshot = screenshot;
        return this;
      }

      /**
       * Convenience method that returns only {@link Boolean#TRUE} or {@link Boolean#FALSE}.
       *
       * <p>
       * Boolean properties can have four possible values:
       * {@code null}, {@link com.google.api.client.util.Data#NULL_BOOLEAN}, {@link Boolean#TRUE}
       * or {@link Boolean#FALSE}.
       * </p>
       *
       * <p>
       * This method returns {@link Boolean#TRUE} if the default of the property is {@link Boolean#TRUE}
       * and it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
       * {@link Boolean#FALSE} is returned if the default of the property is {@link Boolean#FALSE} and
       * it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
       * </p>
       *
       * <p>
       * Indicates if binary data containing a screenshot should be included
       * </p>
       */
      public boolean isScreenshot() {
        if (screenshot == null || screenshot == com.google.api.client.util.Data.NULL_BOOLEAN) {
          return false;
        }
        return screenshot;
      }

      /** The locale used to localize formatted results */
      @com.google.api.client.util.Key
      private String locale;

      /** The locale used to localize formatted results
       */
      public String getLocale() {
        return locale;
      }

      /** The locale used to localize formatted results */
      public Runpagespeed setLocale(String locale) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(LOCALE_PATTERN.matcher(locale).matches(),
              "Parameter locale must conform to the pattern " +
              "[a-zA-Z]+(_[a-zA-Z]+)?");
        }
        this.locale = locale;
        return this;
      }

      /** A Page Speed rule to run; if none are given, all rules are run */
      @com.google.api.client.util.Key
      private java.util.List<String> rule;

      /** A Page Speed rule to run; if none are given, all rules are run
       */
      public java.util.List<String> getRule() {
        return rule;
      }

      /** A Page Speed rule to run; if none are given, all rules are run */
      public Runpagespeed setRule(java.util.List<String> rule) {
        if (!getSuppressPatternChecks()) {
          for (String ruleElem : rule) {
            Preconditions.checkArgument(RULE_PATTERN.matcher(ruleElem).matches(),
                "Parameter rule elements must conform to the pattern " +
                "[a-zA-Z]+");
          }
        }
        this.rule = rule;
        return this;
      }

      /** The analysis strategy to use */
      @com.google.api.client.util.Key
      private String strategy;

      /** The analysis strategy to use
       */
      public String getStrategy() {
        return strategy;
      }

      /** The analysis strategy to use */
      public Runpagespeed setStrategy(String strategy) {
        this.strategy = strategy;
        return this;
      }

      /** Indicates if third party resources should be filtered out before PageSpeed analysis. */
      @com.google.api.client.util.Key("filter_third_party_resources")
      private Boolean filterThirdPartyResources;

      /** Indicates if third party resources should be filtered out before PageSpeed analysis. [default:
     false]
       */
      public Boolean getFilterThirdPartyResources() {
        return filterThirdPartyResources;
      }

      /** Indicates if third party resources should be filtered out before PageSpeed analysis. */
      public Runpagespeed setFilterThirdPartyResources(Boolean filterThirdPartyResources) {
        this.filterThirdPartyResources = filterThirdPartyResources;
        return this;
      }

      /**
       * Convenience method that returns only {@link Boolean#TRUE} or {@link Boolean#FALSE}.
       *
       * <p>
       * Boolean properties can have four possible values:
       * {@code null}, {@link com.google.api.client.util.Data#NULL_BOOLEAN}, {@link Boolean#TRUE}
       * or {@link Boolean#FALSE}.
       * </p>
       *
       * <p>
       * This method returns {@link Boolean#TRUE} if the default of the property is {@link Boolean#TRUE}
       * and it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
       * {@link Boolean#FALSE} is returned if the default of the property is {@link Boolean#FALSE} and
       * it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
       * </p>
       *
       * <p>
       * Indicates if third party resources should be filtered out before PageSpeed analysis.
       * </p>
       */
      public boolean isFilterThirdPartyResources() {
        if (filterThirdPartyResources == null || filterThirdPartyResources == com.google.api.client.util.Data.NULL_BOOLEAN) {
          return false;
        }
        return filterThirdPartyResources;
      }

    }

  }

  /**
   * Builder for {@link Pagespeedonline}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Pagespeedonline}. */
    @Override
    public Pagespeedonline build() {
      return new Pagespeedonline(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link PagespeedonlineRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setPagespeedonlineRequestInitializer(
        PagespeedonlineRequestInitializer pagespeedonlineRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(pagespeedonlineRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
